@@ -4,7 +4,7 @@
  * Created Date: 24/05/2021
  * Author: Shun Suzuki
  * -----
- * Last Modified: 07/09/2021
+ * Last Modified: 02/10/2021
  * Modified By: Shun Suzuki (suzuki@hapis.k.u-tokyo.ac.jp)
  * -----
  * Copyright (c) 2021 Hapis Lab. All rights reserved.
@@ -18,7 +18,7 @@ use crate::{
     gain::Gain,
     geometry::Geometry,
     hardware_defined::{
-        CommandType, RxGlobalControlFlags, RxGlobalHeader, SeqFocus, MOD_FRAME_SIZE,
+        CPUControlFlags, CommandType, FPGAControlFlags, GlobalHeader, SeqFocus, MOD_FRAME_SIZE,
         NUM_TRANS_IN_UNIT,
     },
     modulation::Modulation,
@@ -50,32 +50,35 @@ impl Logic {
 
     pub fn pack_header(
         cmd: CommandType,
-        flag: RxGlobalControlFlags,
+        flag: FPGAControlFlags,
+        cmd_flag: CPUControlFlags,
         data: &mut [u8],
         msg_id: &mut u8,
     ) {
-        let header = data.as_mut_ptr() as *mut RxGlobalHeader;
+        let header = data.as_mut_ptr() as *mut GlobalHeader;
         *msg_id = Self::get_id();
         unsafe {
             (*header).msg_id = *msg_id;
             (*header).ctrl_flag = flag;
-            (*header).mod_size = 0;
             (*header).command = cmd;
+            (*header).command_flag = cmd_flag;
+            (*header).mod_size = 0;
         }
     }
 
     pub fn pack_header_mod<M: Modulation>(
         modulation: &mut M,
-        flag: RxGlobalControlFlags,
+        flag: FPGAControlFlags,
+        cmd_flag: CPUControlFlags,
         data: &mut [u8],
         msg_id: &mut u8,
     ) {
-        Self::pack_header(CommandType::Op, flag, data, msg_id);
+        Self::pack_header(CommandType::Op, flag, cmd_flag, data, msg_id);
         unsafe {
-            let header = data.as_mut_ptr() as *mut RxGlobalHeader;
+            let header = data.as_mut_ptr() as *mut GlobalHeader;
             let mut offset = 0;
             if modulation.sent() == 0 {
-                (*header).ctrl_flag |= RxGlobalControlFlags::MOD_BEGIN;
+                (*header).command_flag |= CPUControlFlags::MOD_BEGIN;
                 (*header).mod_data[0] = (modulation.sampling_frequency_division() & 0xFF) as u8;
                 (*header).mod_data[1] =
                     (modulation.sampling_frequency_division() >> 8 & 0xFF) as u8;
@@ -84,7 +87,7 @@ impl Logic {
             let mod_size = modulation.remaining().clamp(0, MOD_FRAME_SIZE - offset);
             (*header).mod_size = mod_size as u8;
             if modulation.sent() + mod_size >= modulation.buffer().len() {
-                (*header).ctrl_flag |= RxGlobalControlFlags::MOD_END;
+                (*header).command_flag |= CPUControlFlags::MOD_END;
             }
             std::ptr::copy_nonoverlapping(
                 modulation.head(),
@@ -98,10 +101,13 @@ impl Logic {
     pub fn pack_body<G: Gain>(gain: &G, data: &mut [u8], size: &mut usize) {
         let num_devices = gain.data().len();
 
-        *size = std::mem::size_of::<RxGlobalHeader>()
+        *size = std::mem::size_of::<GlobalHeader>()
             + std::mem::size_of::<u16>() * NUM_TRANS_IN_UNIT * num_devices;
         unsafe {
-            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<RxGlobalHeader>());
+            let header = data.as_mut_ptr() as *mut GlobalHeader;
+            (*header).ctrl_flag |= FPGAControlFlags::OUTPUT_ENABLE;
+
+            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>());
             let byte_size = NUM_TRANS_IN_UNIT * std::mem::size_of::<u16>();
             for i in 0..num_devices {
                 std::ptr::copy_nonoverlapping(
@@ -122,17 +128,16 @@ impl Logic {
     ) {
         let num_devices = geometry.num_devices();
 
-        *size = std::mem::size_of::<RxGlobalHeader>()
+        *size = std::mem::size_of::<GlobalHeader>()
             + std::mem::size_of::<u16>() * NUM_TRANS_IN_UNIT * num_devices;
 
-        let header = data.as_mut_ptr() as *mut RxGlobalHeader;
+        let header = data.as_mut_ptr() as *mut GlobalHeader;
         let mut offset = 1;
         unsafe {
-            let mut cursor =
-                data.as_mut_ptr().add(std::mem::size_of::<RxGlobalHeader>()) as *mut u16;
+            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
 
             if seq.sent() == 0 {
-                (*header).ctrl_flag |= RxGlobalControlFlags::SEQ_BEGIN;
+                (*header).command_flag |= CPUControlFlags::SEQ_BEGIN;
                 for i in 0..num_devices {
                     cursor
                         .add(i * NUM_TRANS_IN_UNIT + 1)
@@ -151,7 +156,8 @@ impl Logic {
             );
 
             if seq.sent() + send_size >= seq.size() {
-                (*header).ctrl_flag |= RxGlobalControlFlags::SEQ_END;
+                (*header).command_flag |= CPUControlFlags::SEQ_END;
+                (*header).ctrl_flag |= FPGAControlFlags::OUTPUT_ENABLE;
             }
 
             let fixed_num_unit = 256.0 / geometry.wavelength;
@@ -181,16 +187,15 @@ impl Logic {
     ) {
         let num_devices = geometry.num_devices();
 
-        *size = std::mem::size_of::<RxGlobalHeader>()
+        *size = std::mem::size_of::<GlobalHeader>()
             + std::mem::size_of::<u16>() * NUM_TRANS_IN_UNIT * num_devices;
 
-        let header = data.as_mut_ptr() as *mut RxGlobalHeader;
+        let header = data.as_mut_ptr() as *mut GlobalHeader;
         let seq_sent = *seq.gain_mode() as usize;
         unsafe {
             if seq.sent() == 0 {
-                let cursor =
-                    data.as_mut_ptr().add(std::mem::size_of::<RxGlobalHeader>()) as *mut u16;
-                (*header).ctrl_flag |= RxGlobalControlFlags::SEQ_BEGIN;
+                let cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
+                (*header).command_flag |= CPUControlFlags::SEQ_BEGIN;
                 for i in 0..num_devices {
                     cursor.add(i * NUM_TRANS_IN_UNIT).write(seq_sent as _);
                     cursor
@@ -203,11 +208,11 @@ impl Logic {
             }
 
             if seq.sent() + seq_sent > seq.size() {
-                (*header).ctrl_flag |= RxGlobalControlFlags::SEQ_END;
+                (*header).command_flag |= CPUControlFlags::SEQ_END;
+                (*header).ctrl_flag |= FPGAControlFlags::OUTPUT_ENABLE;
             }
 
-            let mut cursor =
-                data.as_mut_ptr().add(std::mem::size_of::<RxGlobalHeader>()) as *mut u16;
+            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
             let gain_idx = seq.sent() - 1;
 
             match *seq.gain_mode() {
@@ -271,11 +276,10 @@ impl Logic {
         data: &mut [u8],
         size: &mut usize,
     ) {
-        *size = std::mem::size_of::<RxGlobalHeader>()
+        *size = std::mem::size_of::<GlobalHeader>()
             + std::mem::size_of::<u16>() * NUM_TRANS_IN_UNIT * num_devices;
         unsafe {
-            let mut cursor =
-                data.as_mut_ptr().add(std::mem::size_of::<RxGlobalHeader>()) as *mut u16;
+            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
             for delay in delay_offsets {
                 std::ptr::copy_nonoverlapping(delay.as_ptr(), cursor, NUM_TRANS_IN_UNIT);
                 cursor = cursor.add(NUM_TRANS_IN_UNIT);
